// Generated by CoffeeScript 1.8.0
(function() {
  var compile, debug, each, fs, getDependencies, lint, logger, mapCompilerChain, pipeline, sysPath, throwError, waterfall;

  each = require('async-each');

  waterfall = require('async-waterfall');

  debug = require('debug')('brunch:pipeline');

  fs = require('fs');

  sysPath = require('path');

  logger = require('loggy');

  throwError = (function(_this) {
    return function(type, stringOrError) {
      var error, string;
      string = stringOrError instanceof Error ? stringOrError.toString().replace(/^([^:]+:\s+)/, '') : stringOrError;
      error = new Error(string);
      error.code = type;
      return error;
    };
  })(this);

  lint = function(data, path, linters, callback) {
    if (linters.length === 0) {
      return callback(null);
    } else {
      return each(linters, function(linter, callback) {
        return linter.lint(data, path, callback);
      }, callback);
    }
  };

  getDependencies = function(data, path, compiler, callback) {
    if (compiler.getDependencies) {
      debug("getDependencies '" + path + "' with '" + compiler.constructor.name + "'");
      return compiler.getDependencies(data, path, callback);
    } else {
      return callback(null, []);
    }
  };

  mapCompilerChain = function(compiler) {
    return function(params, next) {
      var callback, compiled, compilerArgs, compilerData, dependencies, path, source, sourceMap;
      if (!params) {
        return next();
      }
      dependencies = params.dependencies, compiled = params.compiled, source = params.source, sourceMap = params.sourceMap, path = params.path, callback = params.callback;
      debug("Compiling '" + path + "' with '" + compiler.constructor.name + "'");
      compilerData = compiled || source;
      compilerArgs = compiler.compile.length === 2 ? [
        {
          data: compilerData,
          path: path,
          map: sourceMap
        }
      ] : [compilerData, path];
      compilerArgs.push(function(error, result) {
        if (error != null) {
          return callback(throwError('Compiling', error));
        }
        if (result == null) {
          return next();
        }
        if (toString.call(result) === '[object Object]') {
          if (result.map != null) {
            sourceMap = result.map;
          }
          compiled = result.data;
        } else {
          compiled = result;
        }
        if (compiled == null) {
          throw new Error("Brunch SourceFile: file " + path + " data is invalid");
        }
        return getDependencies(source, path, compiler, (function(_this) {
          return function(error, dependencies) {
            if (error != null) {
              return callback(throwError('Dependency parsing', error));
            }
            return next(null, {
              dependencies: dependencies,
              compiled: compiled,
              source: source,
              sourceMap: sourceMap,
              path: path,
              callback: callback
            });
          };
        })(this));
      });
      return compiler.compile.apply(compiler, compilerArgs);
    };
  };

  compile = function(source, path, compilers, callback) {
    var first;
    first = function(next) {
      return next(null, {
        source: source,
        path: path,
        callback: callback
      });
    };
    return waterfall([first].concat(compilers.map(mapCompilerChain)), callback);
  };

  pipeline = function(path, linters, compilers, callback) {
    debug("Reading '" + path + "'");
    return fs.readFile(path, 'utf-8', function(error, source) {
      if (error != null) {
        return callback(throwError('Reading', error));
      }
      debug("Linting '" + path + "'");
      return lint(source, path, linters, function(error) {
        if (error != null ? error.toString().match(/^warn\:\s/i) : void 0) {
          logger.warn("Linting of " + path + ": " + error);
        } else {
          if (error != null) {
            return callback(throwError('Linting', error));
          }
        }
        return compile(source, path, compilers, callback);
      });
    });
  };

  exports.pipeline = pipeline;

}).call(this);
